export class EventEmitter {
    #listeners = {};
    #isEventListener(listener) {
        return typeof listener === 'function';
    }
    /**
     * Check if there are listeners for event name
     */
    hasListener(eventName) {
        const listenersBundle = this.#listeners[eventName];
        return (listenersBundle && listenersBundle.size > 0);
    }
    /**
     * Emit an event
     * @param eventName Event name
     * @param value Event value that will be passed as an argument to listeners
     */
    emit(...args) {
        const [eventName, value] = args;
        if (!this.hasListener(eventName)) {
            return;
        }
        const listenersBundle = this.#listeners[eventName];
        if (listenersBundle instanceof Map && listenersBundle.size > 0) {
            listenersBundle.forEach(listener => listener(value));
        }
    }
    /**
     * Add multiple listeners for events
     * @param listenersBundle Map of events to listeners
     * @param ref Reference value that can be used to remove the listeners
     */
    on(listenersBundle, ref) {
        for (const eventName in listenersBundle) {
            if (!listenersBundle.hasOwnProperty(eventName)) {
                continue;
            }
            const listener = listenersBundle[eventName];
            if (listener !== undefined &&
                this.#isEventListener(listener)) {
                this.addListener(eventName, listener, ref);
            }
        }
    }
    /**
     * Add a listener to an event
     * @param eventName Event name
     * @param listener Listener function
     * @param ref Reference value that can be used to remove this listener,
     * if omitted it will be the listener itself
     */
    addListener(eventName, listener, ref = listener) {
        if (this.#listeners[eventName] === undefined) {
            this.#listeners[eventName] = new Map();
        }
        this.#listeners[eventName]
            .set(ref, listener);
    }
    /**
     * Remove multiple listeners for events
     * @param eventNames Array of event names
     * @param ref Reference value that map to listeners
     */
    off(eventNames, ref) {
        eventNames.forEach(eventName => this.removeListener(eventName, ref));
    }
    /**
     * Remove a listener to an event
     * @param eventName Event name
     * @param ref Reference value that was used when adding the listener,
     * this can be the listener itself
     */
    removeListener(eventName, ref) {
        const listenersBundle = this.#listeners[eventName];
        if (listenersBundle !== undefined && listenersBundle.has(ref)) {
            listenersBundle.delete(ref);
            if (listenersBundle.size === 0) {
                delete this.#listeners[eventName];
            }
        }
    }
}
//# sourceMappingURL=event-emitter.js.map