import { EventEmitter, Utils } from './';
export class GameEvents extends EventEmitter {
    #features;
    #gameStatus;
    #state;
    #retries;
    #started;
    #startingPromise;
    #onErrorBound;
    #onInfoUpdateBound;
    #onNewEventBound;
    constructor(features, gameStatus) {
        super();
        this.#onErrorBound = this.#onError.bind(this);
        this.#onInfoUpdateBound = this.#onInfoUpdate.bind(this);
        this.#onNewEventBound = this.#onNewEvent.bind(this);
        this.#features = features;
        this.#gameStatus = gameStatus;
        this.#state = {};
        this.#retries = 25;
        this.#started = false;
        this.#startingPromise = null;
    }
    /** Copy of current state, both info updates and last event values */
    get state() {
        return Utils.objectCopy(this.#state);
    }
    /**
     * Call overwolf.games.events.setRequiredFeatures and bind listeners
     * @see https://overwolf.github.io/docs/api/overwolf-games-events#setrequiredfeaturesfeatures-callback
     */
    async start() {
        if (this.#started) {
            return;
        }
        if (this.#startingPromise) {
            return this.#startingPromise;
        }
        this.#startingPromise = this.#start();
        await this.#startingPromise;
        this.#startingPromise = null;
    }
    async #start() {
        this.#stop();
        const success = await this.#setRequiredFeatures();
        if (success) {
            this.#setListeners();
            overwolf.games.events.getInfo(result => this.#onGotInfo(result));
        }
        this.#started = true;
    }
    /** Remove all listeners and clear the {@link state} */
    async stop() {
        if (!this.#started) {
            return;
        }
        if (this.#startingPromise) {
            await this.#startingPromise;
            this.#startingPromise = null;
        }
        this.#stop();
        this.#started = false;
    }
    #stop() {
        for (const key in this.#state) {
            if (this.#state.hasOwnProperty(key)) {
                delete this.#state[key];
            }
        }
        this.#removeListeners();
    }
    /** Remove all listeners and clear the {@link state} */
    destroy() {
        return this.stop();
    }
    #removeListeners() {
        overwolf.games.events.onError.removeListener(this.#onErrorBound);
        overwolf.games.events.onInfoUpdates2.removeListener(this.#onInfoUpdateBound);
        overwolf.games.events.onNewEvents.removeListener(this.#onNewEventBound);
    }
    #setListeners() {
        this.#removeListeners();
        overwolf.games.events.onError.addListener(this.#onErrorBound);
        overwolf.games.events.onInfoUpdates2.addListener(this.#onInfoUpdateBound);
        overwolf.games.events.onNewEvents.addListener(this.#onNewEventBound);
    }
    #onError(err) {
        console.log('GameEvents.#onError()', err);
    }
    #onGotInfo(data) {
        if (!data || !data.success || !data.res) {
            return;
        }
        const info = data.res;
        for (const category in info) {
            if (category === 'features') {
                continue;
            }
            for (const key in info[category]) {
                if (!info[category].hasOwnProperty(key)) {
                    continue;
                }
                const path = `${category}.${key}`;
                let val = info[category][key];
                if (val !== undefined &&
                    (this.#state[path] === undefined || this.#state[path] !== val)) {
                    this.#state[path] = val;
                    try {
                        val = JSON.parse(val);
                    }
                    catch (e) { }
                    this.emit(path, { path, val });
                    this.emit('*', { path, val });
                }
            }
        }
    }
    #onInfoUpdate(data) {
        if (!data || !data.info) {
            return;
        }
        const info = data.info;
        for (const category in info) {
            if (!info.hasOwnProperty(category)) {
                continue;
            }
            for (const key in info[category]) {
                if (!info[category].hasOwnProperty(key)) {
                    continue;
                }
                const path = `${category}.${key}`;
                let val = info[category][key];
                if (val !== undefined &&
                    (this.#state[path] === undefined || this.#state[path] !== val)) {
                    this.#state[path] = val;
                    try {
                        val = JSON.parse(val);
                    }
                    catch (e) { }
                    this.emit(path, { path, val });
                    this.emit('*', { path, val });
                }
            }
        }
    }
    #onNewEvent(data) {
        if (!data.events || !data.events.length)
            return;
        for (const event of data.events) {
            const path = `events.${event.name}`;
            let val = event.data;
            if (val !== undefined &&
                (this.#state[path] === undefined || this.#state[path] !== val)) {
                this.#state[path] = val;
            }
            try {
                val = JSON.parse(val);
            }
            catch (e) { }
            this.emit(path, { path, val });
            this.emit('*', { path, val });
        }
    }
    #tryToSetRequiredFeatures() {
        return new Promise(resolve => {
            overwolf.games.events.setRequiredFeatures(this.#features, resolve);
        });
    }
    async #setRequiredFeatures() {
        let tries = 0, result = null;
        while (tries < this.#retries && this.#gameStatus.isRunning) {
            result = await this.#tryToSetRequiredFeatures();
            if (result.success) {
                console.log(...Utils.L('GameEvents.#setRequiredFeatures(): success:', result));
                return !!(result.supportedFeatures && result.supportedFeatures.length);
            }
            await Utils.delay(2000);
            tries++;
        }
        console.log(`GameEvents.#setRequiredFeatures(): failure after ${tries + 1} tries:`, result);
        return false;
    }
}
//# sourceMappingURL=game-events.js.map