import { EventEmitter } from './event-emitter';
export class GameStatus extends EventEmitter {
    isInFocus;
    isRunning;
    gameInfo;
    #lastGameID;
    #started;
    #startPromise;
    #onGameInfoUpdatedBound;
    constructor() {
        super();
        this.#onGameInfoUpdatedBound = this.#onGameInfoUpdated.bind(this);
        this.isInFocus = false;
        this.isRunning = false;
        this.gameInfo = null;
        this.#lastGameID = null;
        this.#started = false;
        this.#startPromise = null;
    }
    /** Get gameID of current or last running game */
    get gameID() {
        if (this.gameInfo) {
            return Math.floor(this.gameInfo.id / 10);
        }
        return this.#lastGameID;
    }
    async start() {
        if (this.#started) {
            return;
        }
        if (!this.#startPromise) {
            this.#startPromise = this.#start();
        }
        await this.#startPromise;
        this.#started = true;
    }
    async #start() {
        overwolf.games.onGameInfoUpdated.addListener(this.#onGameInfoUpdatedBound);
        const gameInfo = await new Promise(resolve => overwolf.games.getRunningGameInfo(resolve));
        this.#setGameInfo(gameInfo);
        this.isInFocus = !!(gameInfo && gameInfo.isInFocus);
        this.isRunning = !!(gameInfo && gameInfo.isRunning);
    }
    /** Remove all listeners */
    destroy() {
        overwolf.games.onGameInfoUpdated
            .removeListener(this.#onGameInfoUpdatedBound);
        this.#started = false;
    }
    #onGameInfoUpdated(e) {
        const isInFocus = !!(e && e.gameInfo && e.gameInfo.isInFocus), isRunning = !!(e && e.gameInfo && e.gameInfo.isRunning), isInFocusChanged = (isInFocus !== this.isInFocus), isRunningChanged = (isRunning !== this.isRunning);
        if (e && e.gameInfo) {
            this.#setGameInfo(e.gameInfo);
        }
        else {
            this.#setGameInfo(null);
        }
        if (isInFocusChanged) {
            this.isInFocus = isInFocus;
        }
        if (isRunningChanged) {
            this.isRunning = isRunning;
        }
        if (isInFocusChanged) {
            this.emit('focus', isInFocus);
        }
        if (isRunningChanged) {
            this.emit('running', isRunning);
        }
        if (e && e.gameInfo && e.resolutionChanged) {
            this.emit('resolutionChanged', e.gameInfo);
        }
        if (e.gameChanged) {
            this.emit('gameChanged', e.gameInfo || null);
        }
        if (isInFocusChanged ||
            isRunningChanged ||
            e.resolutionChanged ||
            e.gameChanged) {
            this.emit('*', e.gameInfo || null);
        }
    }
    #setGameInfo(gameInfo) {
        if (gameInfo && gameInfo.isRunning) {
            this.gameInfo = gameInfo;
            this.#lastGameID = Math.floor(gameInfo.id / 10);
        }
        else {
            this.gameInfo = null;
        }
    }
}
//# sourceMappingURL=game-status.js.map