import { EventEmitter } from './event-emitter';
import { objectCopy } from './utils';
export class LauncherStatus extends EventEmitter {
    #isInFocus;
    #launcherInfo;
    #started;
    #startingPromise;
    #onLauncherLaunchedBound;
    #onLauncherTerminatedBound;
    #onLauncherInfoUpdatedBound;
    constructor() {
        super();
        this.#onLauncherLaunchedBound = this.#onLauncherLaunched.bind(this);
        this.#onLauncherTerminatedBound = this.#onLauncherTerminated.bind(this);
        this.#onLauncherInfoUpdatedBound = this.#onLauncherInfoUpdated.bind(this);
        this.#launcherInfo = null;
        this.#isInFocus = false;
        this.#started = false;
        this.#startingPromise = null;
    }
    get isRunning() {
        return !!this.#launcherInfo;
    }
    get isInFocus() {
        return this.#isInFocus;
    }
    get launcherInfo() {
        return objectCopy(this.#launcherInfo);
    }
    get launcherID() {
        if (this.#launcherInfo && this.#launcherInfo.classId) {
            return this.#launcherInfo.classId;
        }
        return null;
    }
    async start() {
        if (this.#started) {
            return;
        }
        if (this.#startingPromise) {
            return this.#startingPromise;
        }
        this.#startingPromise = this.#start();
        await this.#startingPromise;
        this.#startingPromise = null;
    }
    async #start() {
        if (this.#started) {
            return;
        }
        if (this.#startingPromise) {
            await this.#startingPromise;
            return;
        }
        this.destroy();
        overwolf.games.launchers.onLaunched
            .addListener(this.#onLauncherLaunchedBound);
        overwolf.games.launchers.onTerminated
            .addListener(this.#onLauncherTerminatedBound);
        overwolf.games.launchers.onUpdated
            .addListener(this.#onLauncherInfoUpdatedBound);
        const launchersInfo = await new Promise(resolve => {
            overwolf.games.launchers.getRunningLaunchersInfo(resolve);
        });
        if (launchersInfo.success &&
            launchersInfo.launchers &&
            launchersInfo.launchers.length) {
            this.#launcherInfo = launchersInfo.launchers[0];
        }
        else {
            this.#launcherInfo = null;
        }
        this.#isInFocus = !!(this.#launcherInfo && this.#launcherInfo.isInFocus);
        this.#started = true;
    }
    /** Remove all listeners */
    destroy() {
        overwolf.games.launchers.onLaunched
            .removeListener(this.#onLauncherLaunchedBound);
        overwolf.games.launchers.onTerminated
            .removeListener(this.#onLauncherTerminatedBound);
        overwolf.games.launchers.onUpdated
            .removeListener(this.#onLauncherInfoUpdatedBound);
        this.#started = false;
    }
    #onLauncherLaunched(info) {
        // console.log('onLauncherLaunched', info)
        const isInFocusChanged = (info.isInFocus !== this.#isInFocus);
        this.#launcherInfo = info;
        if (isInFocusChanged) {
            this.#isInFocus = info.isInFocus;
            this.emit('focus', info.isInFocus);
        }
        this.emit('running', true);
        this.emit('changed', this.launcherInfo);
    }
    #onLauncherTerminated() {
        // console.log('onLauncherTerminated')
        const isInFocusChanged = this.#isInFocus;
        this.#launcherInfo = null;
        this.#isInFocus = false;
        if (isInFocusChanged) {
            this.emit('focus', false);
        }
        this.emit('running', false);
        this.emit('changed', this.launcherInfo);
    }
    #onLauncherInfoUpdated({ info }) {
        // console.log('onLauncherInfoUpdated', info, changeType)
        const isInFocusChanged = (info.isInFocus !== this.#isInFocus);
        this.#launcherInfo = info;
        if (isInFocusChanged) {
            this.#isInFocus = info.isInFocus;
            this.emit('focus', info.isInFocus);
        }
        this.emit('changed', this.launcherInfo);
    }
}
//# sourceMappingURL=launcher-status.js.map