export class OverwolfWindow {
    #name;
    #id;
    constructor(name) {
        this.#name = name;
        this.#id = '';
    }
    async #obtain() {
        const result = await new Promise(resolve => {
            overwolf.windows.obtainDeclaredWindow(this.#name, resolve);
        });
        if (result && result.success && result.window && result.window.id) {
            this.#id = result.window.id;
            return result.window;
        }
        this.#id = '';
        throw new OverwolfWindowError({
            args: [this.#name],
            result
        });
    }
    async restore() {
        await this.#obtain();
        const result = await new Promise(resolve => {
            overwolf.windows.restore(this.#id, resolve);
        });
        if (!result || !result.success) {
            throw new OverwolfWindowError({
                args: [this.#id],
                result
            });
        }
    }
    async minimize() {
        await this.#obtain();
        const result = await new Promise(resolve => {
            overwolf.windows.minimize(this.#id, resolve);
        });
        if (!result || !result.success) {
            throw new OverwolfWindowError({
                args: [this.#id],
                result
            });
        }
    }
    async maximize() {
        await this.#obtain();
        const result = await new Promise(resolve => {
            overwolf.windows.maximize(this.#id, resolve);
        });
        if (!result || !result.success) {
            throw new OverwolfWindowError({
                args: [this.#id],
                result
            });
        }
    }
    async hide() {
        await this.#obtain();
        const result = await new Promise(resolve => {
            overwolf.windows.hide(this.#id, resolve);
        });
        if (!result || !result.success) {
            throw new OverwolfWindowError({
                args: [this.#id],
                result
            });
        }
    }
    async changeSize(width, height, autoDpiResize = true) {
        await this.#obtain();
        const changeSizeParams = {
            window_id: this.#id,
            width,
            height,
            auto_dpi_resize: autoDpiResize
        };
        const result = await new Promise(resolve => {
            overwolf.windows.changeSize(changeSizeParams, resolve);
        });
        if (!result || !result.success) {
            throw new OverwolfWindowError({
                args: [changeSizeParams],
                result
            });
        }
    }
    async changePosition(left, top) {
        await this.#obtain();
        const result = await new Promise(resolve => {
            overwolf.windows.changePosition(this.#id, left, top, resolve);
        });
        if (!result || !result.success) {
            throw new OverwolfWindowError({
                args: [this.#id, left, top],
                result
            });
        }
    }
    async dragMove() {
        await this.#obtain();
        const result = await new Promise(resolve => {
            overwolf.windows.dragMove(this.#id, resolve);
        });
        if (result && result.success) {
            return result;
        }
        throw new OverwolfWindowError({
            args: [this.#id],
            result
        });
    }
    async dragResize(edge) {
        await this.#obtain();
        overwolf.windows.dragResize(this.#id, edge);
    }
    async getWindowState() {
        const result = await new Promise(resolve => {
            overwolf.windows.getWindowState(this.#name, resolve);
        });
        if (result && result.success) {
            return result;
        }
        throw new OverwolfWindowError({
            args: [this.#name],
            result
        });
    }
    async setTopmost(shouldBeTopmost) {
        await this.#obtain();
        const result = await new Promise(resolve => {
            overwolf.windows.setTopmost(this.#id, shouldBeTopmost, resolve);
        });
        if (!result || !result.success) {
            throw new OverwolfWindowError({
                args: [this.#id, shouldBeTopmost],
                result
            });
        }
    }
    async sendToBack() {
        await this.#obtain();
        const result = await new Promise(resolve => {
            overwolf.windows.sendToBack(this.#id, resolve);
        });
        if (!result || !result.success) {
            throw new OverwolfWindowError({
                args: [this.#id],
                result
            });
        }
    }
    async bringToFront(grabFocus) {
        await this.#obtain();
        const result = await new Promise(resolve => {
            overwolf.windows.bringToFront(this.#id, grabFocus, resolve);
        });
        if (!result || !result.success) {
            throw new OverwolfWindowError({
                args: [this.#id, grabFocus],
                result
            });
        }
    }
    async setPosition(properties) {
        await this.#obtain();
        const result = await new Promise(resolve => {
            overwolf.windows.setPosition(this.#id, properties, resolve);
        });
        if (!result || !result.success) {
            throw new OverwolfWindowError({
                args: [this.#id, properties],
                result
            });
        }
    }
    async #close() {
        await this.#obtain();
        await new Promise(resolve => {
            // This resolves even when there's an error throws an error because of
            // a false error in overwolf's close method
            overwolf.windows.close(this.#id, resolve);
        });
    }
    async close() {
        const state = await this.getWindowState();
        if (state.window_state_ex !== 'closed') {
            await this.#close();
        }
    }
    async toggle() {
        const state = await this.getWindowState();
        switch (state.window_state_ex) {
            case 'closed':
            case 'minimized':
            case 'hidden':
                await this.restore();
                break;
            default:
                await this.#close();
                break;
        }
    }
    async setZoom(zoomFactor) {
        await this.#obtain();
        overwolf.windows.setZoom(zoomFactor, this.#id);
    }
    async center() {
        const [win, viewport] = await Promise.all([
            this.#obtain(),
            OverwolfWindow.getViewportSize()
        ]);
        let left = (viewport.width / 2) - (win.width / 2), top = (viewport.height / 2) - (win.height / 2);
        left = Math.max(left, 0);
        top = Math.max(top, 0);
        left = Math.floor(left);
        top = Math.floor(top);
        await this.changePosition(left, top);
    }
    static getWindowsStates() {
        return new Promise(resolve => overwolf.windows.getWindowsStates(resolve));
    }
    static getViewportSize() {
        return new Promise((resolve, reject) => {
            overwolf.utils.getMonitorsList(result => {
                let width = -1, height = -1;
                for (const display of result.displays) {
                    if (display.is_primary) {
                        width = display.width;
                        height = display.height;
                    }
                }
                overwolf.games.getRunningGameInfo(game => {
                    if (game && game.isInFocus) {
                        width = game.logicalWidth;
                        height = game.logicalHeight;
                    }
                    if (width !== -1 && height !== -1) {
                        resolve({ width, height });
                    }
                    else {
                        reject(new Error('Could not get monitors width/height'));
                    }
                });
            });
        });
    }
    static getMonitorsList() {
        return new Promise(resolve => overwolf.utils.getMonitorsList(resolve));
    }
    static async isWindowVisibleToUser() {
        const result = await new Promise(resolve => {
            overwolf.windows.isWindowVisibleToUser(resolve);
        });
        if (result && result.success) {
            return result;
        }
        throw new OverwolfWindowError({ result });
    }
    async centerInViewport(viewport) {
        const win = await this.#obtain();
        let left = viewport.x + (viewport.width / 2) - (win.width / 2), top = viewport.y + (viewport.height / 2) - (win.height / 2);
        left = Math.round(left);
        top = Math.round(top);
        await this.changePosition(left, top);
    }
    static makeViewportWithUid(v) {
        return {
            hash: `${v.x}/${v.y}/${v.width}/${v.height}/${v.scale}`,
            ...v
        };
    }
    static async getGameViewport() {
        return new Promise(resolve => {
            overwolf.games.getRunningGameInfo(game => {
                let viewport = null;
                if (game && game.isRunning) {
                    viewport = OverwolfWindow.makeViewportWithUid({
                        name: game.title,
                        id: game.id,
                        type: "Game" /* Game */,
                        monitorHandle: game.monitorHandle,
                        x: 0,
                        y: 0,
                        width: game.logicalWidth,
                        height: game.logicalHeight,
                        scale: game.width / game.logicalWidth,
                        isPrimary: game.isInFocus,
                        inFocus: game.isInFocus,
                    });
                }
                resolve(viewport);
            });
        });
    }
    static async getViewportsList() {
        const monitors = await OverwolfWindow.getMonitorsList();
        let viewports = monitors.displays
            .map(display => OverwolfWindow.makeViewportWithUid({
            name: display.name,
            id: display.id,
            type: "Monitor" /* Monitor */,
            monitorHandle: display.handle,
            x: display.x,
            y: display.y,
            width: display.width,
            height: display.height,
            scale: display.dpiX / 96,
            isPrimary: display.is_primary,
            inFocus: false
        }));
        const game = await this.getGameViewport();
        if (game) {
            if (game.isPrimary) {
                viewports = viewports.map((viewport) => ({
                    ...viewport,
                    isPrimary: false
                }));
            }
            return [...viewports, game];
        }
        return viewports;
    }
    static async getPrimaryViewport() {
        return (await OverwolfWindow.getViewportsList())
            .find(v => v.isPrimary) || null;
    }
    static async getSecondaryViewport() {
        const viewports = await OverwolfWindow.getViewportsList();
        let viewport = null, viewportAreaSize = 0;
        for (const v of viewports) {
            if (!v.isPrimary &&
                v.type !== "Game" /* Game */ &&
                (v.width * v.height) > viewportAreaSize) {
                viewport = v;
                viewportAreaSize = v.width * v.height;
            }
        }
        return viewport;
    }
}
export class OverwolfWindowError extends Error {
    constructor({ result, args = [] }) {
        let message = JSON.stringify(result);
        if (args && args.length) {
            message += ` args: ${JSON.stringify(args)}`;
        }
        super(message);
        this.name = 'OverwolfWindowError';
        const errConst = Error;
        if (errConst.captureStackTrace) {
            errConst.captureStackTrace(this, OverwolfWindowError);
        }
    }
}
//# sourceMappingURL=overwolf-window.js.map