import { EventEmitter, Utils, WindowTunnel } from '.';
function makeGlobalStateManagerName(name) {
    return `OverwolfAppGlobalStateManager/${name}`;
}
function hydratePersistentState(initialState, prefix) {
    const state = Utils.objectCopy(initialState), prefixLength = prefix.length;
    for (const key in localStorage) {
        if (key.length > prefixLength && key.startsWith(prefix)) {
            const stateKey = key.substr(prefixLength), stored = localStorage.getItem(key);
            if (typeof stored === 'string' && stored !== 'undefined') {
                state[stateKey] = JSON.parse(stored);
            }
        }
    }
    return state;
}
export class StateManager extends EventEmitter {
    globalName;
    #persistent;
    #localStoragePrefix;
    #state;
    constructor(globalName, initialState, persistent = false) {
        super();
        this.globalName = globalName;
        this.#persistent = persistent;
        this.#localStoragePrefix = `${globalName}/`;
        if (this.#persistent) {
            this.#state = hydratePersistentState(initialState, this.#localStoragePrefix);
        }
        else {
            this.#state = Utils.objectCopy(initialState);
        }
        WindowTunnel.set(makeGlobalStateManagerName(globalName), this);
    }
    has(key) {
        return this.#state[key] !== undefined;
    }
    get(key) {
        const value = this.#state[key];
        if (typeof value === 'object' && value !== null) {
            return Utils.objectCopy(value);
        }
        return value;
    }
    #setObject(key, value) {
        const stringified = JSON.stringify(value);
        if (this.#persistent) {
            localStorage.setItem(this.#localStoragePrefix + key, stringified);
        }
        const copy = JSON.parse(stringified);
        this.#state[key] = copy;
        this.emit(key, copy);
    }
    #setPrimitive(key, value) {
        if (this.#persistent) {
            localStorage.setItem(this.#localStoragePrefix + key, JSON.stringify(value));
        }
        this.#state[key] = value;
        this.emit(key, value);
    }
    set(key, value) {
        if (typeof value === 'object' && value !== null) {
            this.#setObject(key, value);
        }
        else {
            this.#setPrimitive(key, value);
        }
    }
    replaceState(state) {
        this.#state = Utils.objectCopy(state);
    }
    clearPersistentState() {
        for (const key in this.#state) {
            if (this.#state.hasOwnProperty(key)) {
                localStorage.removeItem(this.#localStoragePrefix + key);
            }
        }
    }
}
/** Read-only wrapper of the StateManager */
export class StateClient {
    globalName;
    #stateManager;
    constructor(globalName) {
        this.globalName = globalName;
        this.#stateManager = WindowTunnel.get(makeGlobalStateManagerName(globalName));
        if (!this.#stateManager) {
            throw new Error('StateManager server not defined in background (main) window');
        }
        this.addListener = this.#stateManager.addListener.bind(this.#stateManager);
        this.on = this.#stateManager.on.bind(this.#stateManager);
        this.off = this.#stateManager.off.bind(this.#stateManager);
        this.removeListener =
            this.#stateManager.removeListener.bind(this.#stateManager);
        this.has = this.#stateManager.has.bind(this.#stateManager);
        this.get = this.#stateManager.get.bind(this.#stateManager);
    }
    addListener;
    on;
    off;
    removeListener;
    has;
    get;
}
export function makeNiceState(manager) {
    const proxyHandler = {
        has(target, key) {
            return target.has(key);
        },
        get(target, key) {
            switch (key) {
                case 'on':
                    return target.on.bind(target);
                case 'off':
                    return target.off.bind(target);
                case 'addListener':
                    return target.addListener.bind(target);
                case 'removeListener':
                    return target.removeListener.bind(target);
                default:
                    return target.get(key);
            }
        },
        set(target, key, value) {
            switch (key) {
                case 'on':
                case 'off':
                case 'addListener':
                case 'removeListener':
                    throw new Error(`"${key}" is a reserved property for State`);
            }
            target.set(key, value);
            return true;
        },
        deleteProperty() {
            throw new Error('Deleting properties is not allowed for State');
        },
        ownKeys() {
            return [];
        }
    };
    return new Proxy(manager, proxyHandler);
}
export function makeNiceStateClient(globalName) {
    const client = new StateClient(globalName);
    const proxyHandler = {
        has(target, key) {
            return target.has(key);
        },
        get(target, key) {
            switch (key) {
                case 'on':
                    return target.on.bind(target);
                case 'off':
                    return target.off.bind(target);
                case 'addListener':
                    return target.addListener.bind(target);
                case 'removeListener':
                    return target.removeListener.bind(target);
                default:
                    return target.get(key);
            }
        },
        set() {
            throw new Error('Setting properties is not allowed for StateClient');
        },
        deleteProperty() {
            throw new Error('Deleting properties is not allowed for StateClient');
        },
        ownKeys() {
            return [];
        }
    };
    return new Proxy(client, proxyHandler);
}
//# sourceMappingURL=state.js.map