import { EventEmitter, Utils, WindowTunnel } from './';
function hydratePersistentState(initialState, prefix) {
    const state = Utils.objectCopy(initialState), prefixLength = prefix.length;
    for (const key in localStorage) {
        if (key.length > prefixLength && key.startsWith(prefix)) {
            const stateKey = key.substr(prefixLength);
            const stored = localStorage.getItem(key);
            if (typeof stored === 'string' && stored !== 'undefined') {
                state[stateKey] = JSON.parse(stored);
            }
        }
    }
    return state;
}
class StateManager extends EventEmitter {
    name;
    // readonly #initialState: StateMap
    #state;
    #persistent;
    #localStoragePrefix;
    constructor(name, initialState, persistent = false) {
        super();
        this.name = name;
        this.#persistent = persistent;
        this.#localStoragePrefix = `${name}/`;
        // this.#initialState = initialState;
        if (this.#persistent) {
            this.#state = hydratePersistentState(initialState, this.#localStoragePrefix);
        }
        else {
            this.#state = Utils.objectCopy(initialState);
        }
    }
    has(key) {
        return (this.#state[key] !== undefined);
    }
    get(key) {
        const value = this.#state[key];
        if (typeof value === 'object' && value !== null) {
            return Utils.objectCopy(value);
        }
        return value;
    }
    #setObject(key, value) {
        const stringified = JSON.stringify(value);
        if (this.#persistent) {
            localStorage.setItem(this.#localStoragePrefix + key, stringified);
        }
        const copy = JSON.parse(stringified);
        this.#state[key] = copy;
        this.emit(key, copy);
    }
    #setPrimitive(key, value) {
        if (this.#persistent) {
            localStorage.setItem(this.#localStoragePrefix + key, JSON.stringify(value));
        }
        this.#state[key] = value;
        this.emit(key, value);
    }
    set(key, value) {
        if (typeof value === 'object' && value !== null) {
            this.#setObject(key, value);
        }
        else {
            this.#setPrimitive(key, value);
        }
    }
    replaceState(state) {
        this.#state = Utils.objectCopy(state);
    }
    clearPersistentState() {
        for (const key in this.#state) {
            if (this.#state.hasOwnProperty(key)) {
                localStorage.removeItem(this.#localStoragePrefix + key);
            }
        }
    }
}
function makeGlobalStateName(name) {
    return `GlobalState/${name}`;
}
export function makeState(name, initialState, persistent = false) {
    const manager = new StateManager(name, initialState, persistent);
    const store = new Proxy(manager, {
        has(target, key) {
            return target.has(key);
        },
        get(target, key) {
            switch (key) {
                case 'on':
                    return target.on.bind(target);
                case 'off':
                    return target.off.bind(target);
                case 'addListener':
                    return target.addListener.bind(target);
                case 'removeListener':
                    return target.removeListener.bind(target);
                default:
                    return target.get(key);
            }
        },
        set(target, key, value) {
            switch (key) {
                case 'on':
                case 'off':
                case 'addListener':
                case 'removeListener':
                    throw new Error(`${key} is a reserved property for Store`);
            }
            target.set(key, value);
            return true;
        },
        deleteProperty() {
            return false;
        },
        ownKeys() {
            return [];
        }
    });
    WindowTunnel.set(makeGlobalStateName(name), store);
    return store;
}
export function makeStateClient(name) {
    return WindowTunnel.get(makeGlobalStateName(name));
}
//# sourceMappingURL=store.js.map